!  FastMode.f90 
!  Copyright (c) 1997-2002 Zeal SoftStudio.
!  E-Mail: support@zealsoftstudio.com
!  Web: http://www.zealsoftstudio.com
!  This sample illustrates how to use the fast mode of
!  NTPort Library to increase performance.
!
!  FUNCTIONS:
!   WinMain()      - Entry point for the application;
!                    displays the main window; processes the message loop
!   FastModeSub()  - Callback routine for the main dialog box
!   FastModeApply()- Callback routine for the APPLY button
!

!****************************************************************************
!
!  FUNCTION: WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!
!  PURPOSE:  Entry point for the application
!
!  COMMENTS: Displays the main window and processes the message loop
!
!****************************************************************************

include '../ntport.f90'

integer*4 function WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'WinMain' :: WinMain
!DEC$ ENDIF

    use user32
    use kernel32
    use dflogm
    use FastModeGlobals

    implicit none

    integer*4 hInstance
    integer*4 hPrevInstance
    integer*4 lpszCmdLine
    integer*4 nCmdShow

    include 'resource.fd'

    external FastModeSub
    external FastModeApply
	external FastMode_NormalTest
	external FastMode_FastTest

    ! Variables
    type (T_MSG)            mesg
    integer*4               ret
    logical*4               lret


    ghInstance = hInstance
    ghModule = GetModuleHandle(NULL)
    ghwndMain = NULL

    lret = DlgInit(IDD_FASTMODE_DIALOG, gdlg)
    if (lret == .FALSE.) goto 99999
    lret = DlgSetSub(gdlg, IDD_FASTMODE_DIALOG, FastModeSub)
    lret = DlgSetSub(gdlg, IDM_APPLY, FastModeApply)
	lret = DlgSetSub(gdlg, IDM_NormalMode, FastMode_NormalTest)
	lret = DlgSetSub(gdlg, IDM_FastMode, FastMode_FastTest)
	lret = DlgSet(gdlg , IDC_EDITAddress ,'200')
	lret = DlgSet(gdlg , IDC_EDITTime ,'10000')

    lret = DlgModeless(gdlg, nCmdShow)
    if (lret == .FALSE.) goto 99999

    ! Read and process messsages
    do while( GetMessage (mesg, NULL, 0, 0) ) 
	   if ((mesg.message .EQ. WM_COMMAND ).AND.( mesg.wParam .EQ. 2))then
	      return 
	   end if
       if ( DlgIsDlgMessage(mesg) .EQV. .FALSE. ) then
           lret = TranslateMessage( mesg )
           ret  = DispatchMessage( mesg )
       end if
    end do
    call DlgUninit(gdlg)

    WinMain = mesg.wParam
    return

99999 &

    ret = MessageBox(ghwndMain, "Error initializing application FastMode"C, &
                     "Error"C, MB_OK)
    WinMain = 0

end 

!****************************************************************************
!
!  FUNCTION: FastModeSub ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for initialization and destroy
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE FastModeSub( dlg, id, callbacktype )

  use user32
  use dflogm

  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_destroy) then
    call PostQuitMessage(0)
  endif

  END SUBROUTINE FastModeSub

!****************************************************************************
!  FUNCTION: FastModeApply ( dlg, id, callbacktype )
!  PURPOSE:  Dialog box callback for APPLY button
!  COMMENTS:
!****************************************************************************
SUBROUTINE FastModeApply( dlg, id, callbacktype )

  use dflogm

  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_clicked) then
    ! TO DO; Add your APPLY logic here
  endif

  END SUBROUTINE FastModeApply


!****************************************************************************
!  subroutine: TestMode ( dlg, bFastMode )
!  PURPOSE:  subroutine for test the speed of reading data
!  COMMENTS:
!****************************************************************************
SUBROUTINE TestMode( dlg,  bFastMode, id)

  use dflogm
  use ntport
  use user32
  use kernel32
  USE DFPORT


  implicit none
  include 'resource.fd'
  type (dialog) dlg
  integer id, callbacktype
  logical bFastMode


  integer PortAddress , nData
  character (len=100) text ,textstate
  logical lret 
  integer i ,m_nTimes 
  real(8) tTime

  if ( bFastMode ) then
    ! TO DO; Add your APPLY logic here
    ! Using Fast mode
	lret = IsWinNT()
	if(lret  .NE. .TRUE.) then 
		lret = MessageBox(NULL, 'Fast mode is designed for Windows NT/2000, not Windows 95/98'C, 'Error', MB_OK)

		goto 300 
	end if
    call SetFastMode(bFastMode)
  end if

    ! Get the address of the port and times
		    lret = DlgGet(dlg,IDC_EDITAddress,text)
			read (text, 100) PortAddress
100         format(Z8)
		    lret = DlgGet(dlg,IDC_EDITTime,text)
			read (text, *) m_nTimes     

	!To use the fast mode, you must call EnablePorts
    !before you call the Inport, Outport, and other functions.
	if (bFastMode .EQ. .TRUE.) then
	    call EnablePorts(PortAddress,PortAddress)  !under NT system,call enableport first when use fast mode
    	lret = GetLastState(textstate) 
		if (lret .NE. ERROR_NONE) then
			 lret = MessageBox(NULL , textstate,'Error', MB_ICONSTOP)
			 goto 300
		end if
	end if
    nData = Inp(PortAddress)

    if( GetLastState(textstate) .NE. ERROR_NONE) then
    
        lret = MessageBox(NULL,textstate,'Error', MB_ICONSTOP)
		goto 300
	end if

		! Start the time count
		tTime = TIMEF( )

		!read data 
		do i = 1 , m_nTimes
			nData = Inp(PortAddress)
		end do
		if (bFastMode .EQ. .TRUE.) then
			call DisablePorts(PortAddress, PortAddress)
		end if
    	tTime = (TIMEF( ) -tTime )*1000

		!show the result
		write(text,200)tTime
200     format("time cost of reading data : ",F10.3 ,"ms")
		lret = DlgSet(dlg,id,text)
  
             
300    i=1 !return  

  END SUBROUTINE TestMode


!****************************************************************************
!  FUNCTION: FastMode_NormalTest ( dlg, id, callbacktype )
!  PURPOSE:  Dialog box callback for FastMode_NormalTest button
!  COMMENTS:
!****************************************************************************
SUBROUTINE FastMode_NormalTest( dlg, id, callbacktype )

  use dflogm
  use ntport


  implicit none
  include 'resource.fd'
  type (dialog) dlg
  integer id, callbacktype


  if (callbacktype == dlg_clicked) then
    ! TO DO; Add your APPLY logic here
    call TestMode( dlg,  .FALSE., IDC_STATICNormal)
    

  endif

  END SUBROUTINE FastMode_NormalTest

  !****************************************************************************
!  FUNCTION: FastMode_FastTest ( dlg, id, callbacktype )
!  PURPOSE:  Dialog box callback for FastMode_FastTest button
!  COMMENTS:
!****************************************************************************
SUBROUTINE FastMode_FastTest( dlg, id, callbacktype )


    use ntport
    use dflogm


  implicit none
  include 'resource.fd'
  type (dialog) dlg
  integer id, callbacktype

  character (len=100) text


  if (callbacktype == dlg_clicked) then
    ! TO DO; Add your APPLY logic here
		 call TestMode( dlg,  .TRUE., IDC_STATICFast)
  endif

  END SUBROUTINE FastMode_FastTest