!  Speaker.f90 
!  Copyright (c) 1997-2002 Zeal SoftStudio.
!  E-Mail: support@zealsoftstudio.com
!  Web: http://www.zealsoftstudio.com
!  This sample illustrates how to use NTPort
!  Library to control PC speaker.
!
!  FUNCTIONS:
!   WinMain()      - Entry point for the application;
!                    displays the main window; processes the message loop
!   SpeakerSub()  - Callback routine for the main dialog box
!   SpeakerApply()- Callback routine for the APPLY button
!

!****************************************************************************
!
!  FUNCTION: WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!
!  PURPOSE:  Entry point for the application
!
!  COMMENTS: Displays the main window and processes the message loop
!
!****************************************************************************
include '../ntport.f90'
integer*4 function WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'WinMain' :: WinMain
!DEC$ ENDIF

    use user32
    use kernel32
    use dflogm
    use SpeakerGlobals

    implicit none

    integer*4 hInstance
    integer*4 hPrevInstance
    integer*4 lpszCmdLine
    integer*4 nCmdShow

    include 'resource.fd'

    external SpeakerSub
    external SpeakerApply
	external SpeakerOff
	external SpeakerOn

    ! Variables
    type (T_MSG)            mesg
    integer*4               ret
    logical*4               lret


    ghInstance = hInstance
    ghModule = GetModuleHandle(NULL)
    ghwndMain = NULL

    lret = DlgInit(IDD_SPEAKER_DIALOG, gdlg)
    if (lret == .FALSE.) goto 99999
    lret = DlgSetSub(gdlg, IDD_SPEAKER_DIALOG, SpeakerSub)
    lret = DlgSetSub(gdlg, IDM_APPLY, SpeakerApply)

    lret = DlgSetSub(gdlg, IDM_SoundOff, SpeakerOff)
    lret = DlgSetSub(gdlg, IDM_SoundOn, SpeakerOn)
	lret = DlgSet (gdlg, IDC_EDIT1, '262')


    lret = DlgModeless(gdlg, nCmdShow)
    if (lret == .FALSE.) goto 99999

    ! Read and process messsages
    do while( GetMessage (mesg, NULL, 0, 0) ) 
	   if ((mesg.message .EQ. WM_COMMAND ).AND.( mesg.wParam .EQ. 2))then
	      return 
	   end if
       if ( DlgIsDlgMessage(mesg) .EQV. .FALSE. ) then
           lret = TranslateMessage( mesg )
           ret  = DispatchMessage( mesg )
       end if
    end do
    call DlgUninit(gdlg)

    WinMain = mesg.wParam
    return

99999 &

    ret = MessageBox(ghwndMain, "Error initializing application Speaker"C, &
                     "Error"C, MB_OK)
    WinMain = 0

end 

!****************************************************************************
!
!  FUNCTION: SpeakerSub ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for initialization and destroy
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE SpeakerSub( dlg, id, callbacktype )

  use user32
  use dflogm

  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_destroy) then
    call PostQuitMessage(0)
  endif

  END SUBROUTINE SpeakerSub



!****************************************************************************
!  FUNCTION: Speaker (bOn)
!  PURPOSE:  set sound on or off
!  COMMENTS: set the speaker on when bOn is equal to true,otherwise set speaker off 
!****************************************************************************

SUBROUTINE Speaker(dlg , bOn )

  use dflogm
  use ntport   !To use the NTportlibrary

  implicit none
  type (dialog) dlg
   logical bOn
   include 'resource.fd'


    integer  nValue
	integer nDivider , Clk8253 ,nFreq
	logical lret
	CHARACTER(256) text	

    nFreq= 0
    nValue = Inport( Z'61' )
	Clk8253 = 1193180
    if (bOn) then
        nValue = Z'2F'
	    !get frequency value
	    lret = DlgGet(dlg,IDC_EDIT1,text)
		if (text .GT. '0') then
				read (text, *) nFreq
		else
			nFreq = 262
		end if


	   !set frequency
	   	nDivider =  Clk8253 / nFreq
		call Outport(Z'43', Z'b6');		! timer 2, square wave
		call Outport(Z'42', MOD(nDivider , 256))
		call Outport(Z'42', nDivider / 256)
		
	else
        nValue = Z'2c'
	end if

    call Outport(Z'61', nValue)
	lret = GetLastState(text)
	lret = DlgSet(dlg , IDC_STATICshow ,text)


  END SUBROUTINE Speaker

!****************************************************************************
!
!  FUNCTION: SpeakerApply ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for APPLY button
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE SpeakerApply( dlg, id, callbacktype )

  use dflogm
  use ntport

  implicit none

  type (dialog) dlg
  integer id, callbacktype
   integer  nValue
  if (callbacktype == dlg_clicked) then
    ! TO DO; Add your APPLY logic here
	call Speaker(dlg , .FALSE.)
	call DlgExit(dlg)

  endif

  END SUBROUTINE SpeakerApply


!****************************************************************************
!
!  FUNCTION: SpeakerOn ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for set sound on button
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE SpeakerOn( dlg, id, callbacktype )

  use dflogm

  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_clicked) then
    ! TO DO; Add your APPLY logic here
	call Speaker(dlg,.TRUE.)
  end if

  END SUBROUTINE SpeakerOn

!****************************************************************************
!
!  FUNCTION: SpeakerOff ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for set sound off button
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE SpeakerOff( dlg, id, callbacktype )

  use dflogm

  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_clicked) then
    ! TO DO; Add your APPLY logic here
	call Speaker(dlg , .FALSE.)
  end if

  END SUBROUTINE SpeakerOff